#![warn(
    clippy::all,
/*    clippy::restriction,*/
    clippy::pedantic,
    clippy::nursery,
    clippy::cargo
)]
#![allow(clippy::non_ascii_literal)]

use log::warn;

pub struct UserDBLocal<'a> {
    pub passwd_entries: Vec<crate::User<'a>>,
    pub shadow_entries: Vec<crate::Shadow<'a>>,
    pub group_entries: Vec<crate::Group<'a>>,
}

impl<'a> UserDBLocal<'a> {
    #[must_use]
    pub fn import_from_strings(
        passwd_content: &'a str,
        shadow_content: &'a str,
        group_content: &'a str,
    ) -> Self {
        let res = UserDBLocal {
            passwd_entries: passwd_content
                .lines()
                .filter_map(|line| {
                    if line.len() > 5 {
                        println!("{}", line);
                        Some(crate::User::new_from_string(line).expect("failed to read lines"))
                    } else {
                        None
                    }
                })
                .collect(),
            group_entries: group_content
                .lines()
                .filter_map(|line| {
                    if line.len() > 5 {
                        Some(crate::Group::new_from_string(line).expect("Parsing failed"))
                    } else {
                        None
                    }
                })
                .collect(),
            shadow_entries: shadow_content
                .lines()
                .filter_map(|line| {
                    if line.len() > 5 {
                        Some(crate::Shadow::new_from_string(line).expect("Parsing failed"))
                    } else {
                        None
                    }
                })
                .collect(),
        };
        res
    }
}

#[test]
fn test_creator_user_db_local() {
    let data = UserDBLocal::import_from_strings("testuser:x:1001:1001:full Name,004,000342,001-2312,myemail@test.com:/home/test:/bin/test", "test:!!$6$/RotIe4VZzzAun4W$7YUONvru1rDnllN5TvrnOMsWUD5wSDUPAD6t6/Xwsr/0QOuWF3HcfAhypRkGa8G1B9qqWV5kZSnCb8GKMN9N61:18260:0:99999:7:::", "teste:x:1002:test,teste");
    assert_eq!(
        data.passwd_entries.get(0).unwrap().get_username(),
        "testuser"
    )
}

#[test]
fn test_parsing_local_database() {
    use std::fs::File;
    use std::io::{BufReader, Read};
    let passwd_file = File::open("/etc/passwd").unwrap();
    let mut passwd_reader = BufReader::new(passwd_file);
    let mut my_passwd_lines = "".to_string();
    passwd_reader.read_to_string(&mut my_passwd_lines).unwrap();
    let group_file = File::open("/etc/group").unwrap();
    let mut group_reader = BufReader::new(group_file);
    let mut my_group_lines = "".to_string();
    group_reader.read_to_string(&mut my_group_lines).unwrap();
    let data = UserDBLocal::import_from_strings(&my_passwd_lines, "", &my_group_lines);
    assert_eq!(data.group_entries.get(0).unwrap().get_groupname(), "root");
}
